import {
  APICreate,
  APIListPaginated,
  APIRemove,
  APIRetrieve,
  APIUpdate,
  createExtendableAPI,
} from '@rokoli/bnb/drf'
import { defineStore } from 'pinia'

import { createSteeringURL } from '@/api'
import { steeringAuthInit } from '@/stores/auth'
import { Schedule, ScheduleConflictSolution, ScheduleCreateData } from '@/types'
import { computed } from 'vue'

export const useScheduleStore = defineStore('schedules', () => {
  const endpoint = createSteeringURL.prefix('schedules')
  const { api, base } = createExtendableAPI<Schedule>(endpoint, steeringAuthInit)
  const { update } = APIUpdate<Schedule, ScheduleCreateData>(api)
  return {
    ...base,
    update,
    ...APICreate<Schedule, ScheduleCreateData>(api),
    ...APIListPaginated(api),
    ...APIRetrieve(api),
    ...APIRemove(api),
  }
})

export function useConflictSolutions(translate: (v: string) => string) {
  const t = translate
  const modes: readonly ScheduleConflictSolution[] = Object.freeze([
    'ours',
    'theirs',
    'theirs-start',
    'theirs-end',
    'theirs-both',
    'ours-start',
    'ours-end',
    'ours-both',
  ])
  const modeChoices = computed<{ mode: ScheduleConflictSolution; label: string }[]>(() =>
    modes.map((mode) => ({ mode, label: t(`conflictResolution.strategies.${mode}`) })),
  )

  return { modes, modeChoices }
}

export type RepetitionType = 'next-day' | 'next-business-day' | 'n-days-later'

export function useRepetitionRules(translate: (v: string) => string) {
  const t = translate
  return computed<
    { id: RepetitionType; label: string; defaults: Partial<ScheduleCreateData['schedule']> }[]
  >(() => [
    {
      id: 'next-day',
      label: t('repetition.nextDay'),
      defaults: {
        addBusinessDaysOnly: false,
        addDaysNo: 1,
      },
    },
    {
      id: 'next-business-day',
      label: t('repetition.nextBusinessDay'),
      defaults: {
        addBusinessDaysOnly: true,
        addDaysNo: 1,
      },
    },
    {
      id: 'n-days-later',
      label: t('repetition.nDaysLater'),
      defaults: {},
    },
  ])
}
